#include "GLUTcallbacks.h"

extern Scene *scene;     /* defined in mainMaze.cpp */
extern FrameBuffer fb;   /* defined in mainMaze.cpp */
extern bool opengl_test; /* defined in mainMaze.cpp */
extern bool bilerp_flag; /* defined in mainMaze.cpp */
extern Timer timer1;     /* defined in mainMaze.cpp */


void display(void)
{
   double time;
   char name[128];

   timer1.startTimer();

   /* first set up the matrices */
   computeViewMatrix();         /* in GLUTMouseCallbacks.cpp */
   computeProjectionMatrix();   /* in GLUTMouseCallbacks.cpp */

   if (opengl_test)
   {
      /* draw the scene with opengl */
      renderSceneOpenGL(scene);
   }
   else
   {
      /* draw the scene with software */
      fb.renderScene(scene);
      glDrawPixels(fb.width, fb.height, GL_RGB, GL_UNSIGNED_BYTE, fb.pixel_buffer);
   }

   glFinish(); /* wait while everything gets rendered */

   timer1.stopTimer();
   time = timer1.getTime();
   sprintf(name, "%s: %.4lf s, %.2lf fps", opengl_test ? "GL" : "soft", time, 1.0/time);
   glutSetWindowTitle(name);

   glutSwapBuffers();
   return;
}


void redisplay(void)
{
   glutPostRedisplay();
   return;
}


void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
      case 27:
      case 'q':
      case 'Q':
         /* quit the program */
         exit(0);
         break;
      case ' ':
         /* toggle between opengl and sofware rendering */
         opengl_test = !opengl_test;
         redisplay();
         break;
      case 'w':
         moveForward();
         redisplay();
         break;
      case 'a':
         moveLeft();
         redisplay();
         break;
      case 'd':
         moveRight();
         redisplay();
         break;
      case 'z':
         moveBack();
         redisplay();
         break;
      case 'b':
         /* toggle bilinear interpolation of the textures */
         bilerp_flag = !bilerp_flag;
         switchTextureFilteringOpenGL(scene);
         if (bilerp_flag)
         {
            fprintf(stderr,"glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)\n");
         }
         else
         {
            fprintf(stderr, "glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)\n");
         }
         fflush(stderr);
         redisplay();
      default:
         break;
   }
   return;
}
