#ifndef MOUSE_H
#define MOUSE_H

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "gl.h"
#include "glu.h"
#include "glut.h"
#include "Constants.h"

#include "SoftwareRenderer.h"

#define FLOAT_EPSILON 0.0000001


/* three GLUT callback functions */
void reshape(int w, int h);
void mouseMotion(int x, int y);
void mouseInput(int button, int state, int x, int y);


/* this function establishes the projection matrix
   using either openGL calls or
   software calls which you will fill in */
void computeProjectionMatrix(void);

/* this function establishes the view matrix
   using either openGL calls or
   software calls which you will fill in */
void computeViewMatrix(void);


void beginDrag(int x, int y);
void dragView(int x, int y);

void beginZoom(int x, int y);

/* these rotations all assume the right-hand rule,
   so if the vector is pointing towards the viewer,
   the positive rotation is counterclockwise */
void rotateAboutX(float *vector, float angle);
void rotateAboutY(float *vector, float angle);
void rotateAboutZ(float *vector, float angle);

void beginTranslate(int x, int y);
void addVector(float *res_vector, float *vector1, float *vector2);
float dotProduct(float *v1, float *v2);
void scaleVector(float *new_vector, float *vector, float scale);
void crossProduct(float *res_vector, float *vector1, float *vector2);


/* this function computes some useful vectors to determine where the
   viewer is facing.  this is used in the move functions, since you
   move in the direction you are facing, for example */
void computeUpSightAndRightVectors(void);


/* functions to move the viewpoint around */
void moveForward(void);
void moveLeft(void);
void moveRight(void);
void moveBack(void);


bool floatEqual(float a, float b);

#endif  /* MOUSE_H */
