#include <stdio.h>
#include "Constants.h"
#include "SoftwareRenderer.h"
#include "MazeParser.h"
#include "GLUTCallbacks.h"
#include "GLUTMouseCallbacks.h"


/*** Define global variables ***/
Scene *scene; /* Pointer to the Scene object that we will build and then render. */
FrameBuffer fb(window_width, window_height); /* This is the Framebuffer object that
                                                the software renderer will render
                                                the Scene object into. */
bool opengl_test = false; /* Determines OpenGL or software rendering mode */
bool bilerp_flag = false; /* Determines which rendering mode to use for textures. */
Timer timer1;             /* Used to compute rendering speed in frames per second */

/* store the eye position */
float eye_fov;    /* field of view of the eye */
float eye_theta;  /* the theta angle of the eye (the longitude).
                     it is positive in a counterclockwise direction when looking down -z */
float eye_phi;    /* the phi angle of the eye (the latitude).
                     i define 0 as the north pole. */
float eye_pos[3]; /* the position of the eye,
                     computed from eye_theta, eye_phi, eye_distance, and eye_origin */

float sight_vector[3];  /* vector in the direction of the eye */
float up_vector[3];
float right_vector[3];

/*** end of global variables ***/


/* initialize the eye parameters and the mouse */
void initMouse(void)
{
   /* first initialize the eye parameters */
   eye_theta = 45;
   eye_phi = 0;

   eye_fov = 45;

   /* set the eye origin to (0,0,0) */
   eye_pos[0] = 0;
   eye_pos[1] = 0;
   eye_pos[2] = 5;

   computeUpSightAndRightVectors();   /* in GLUTMouseCallbacks.cpp */

   /* more glut callback functions */
   glutReshapeFunc(reshape);          /* in GLUTMouseCallbacks.cpp */
   glutMotionFunc(mouseMotion);       /* in GLUTMouseCallbacks.cpp */
   glutPassiveMotionFunc(mouseMotion);
   glutMouseFunc(mouseInput);         /* in GLUTMouseCallbacks.cpp */

   return;
}


void initGLUT(int argc, char **argv)
{
   /* Initialize GLUT */
   glutInit(&argc, argv);
   glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
   glutInitWindowSize(window_width, window_height);
   glutCreateWindow("window_name");
   glutInitWindowPosition(100,50);

   glutDisplayFunc(display);    /* in GLUTCallbacks.cpp */
   glutKeyboardFunc(keyboard);  /* in GLUTCallbacks.cpp */
   glutIdleFunc(redisplay);     /* in GLUTCallbacks.cpp */
 //glutIdleFunc(NULL);

   initMouse(); /* see above */

   return;
}


int main(int argc, char **argv)
{
   if (argc != 2)
   {
      fprintf(stderr,"Usage: MazeViewer <file>\n");
      fprintf(stderr,"argc: %d\n", argc);
      return -1;
   }

   initGLUT(argc, argv); /* see above */

   /* Create the Scene object that we shall render. */
   MazeParser mp;
   scene = mp.loadScene(argv[1]);

   initializeTexturesOpenGL(scene); /* in RenderOpenGL.cpp */

   glutMainLoop();                  /* Enter GLUT's main loop */

   return 1;
}
